#!/usr/bin/env bash

POINTS=5

CFLAGS="-O2 -Wall" make -C src myspin mysplit mystop myint
CFLAGS="-O2 -Wall" make handin
mkdir -p sub
mkdir -p ref
pushd src
for trace in ../traces/trace*.txt
do
  echo "Running ${trace}"
  ./sdriver.pl -t ${trace} -s ../handin -g > ${trace/traces/sub} 2>&1
  ./sdriver.pl -t ${trace} -s ./tsh -g > ${trace/traces/ref} 2>&1
done
popd



correct=0



for num in {01..10}
do
  echo "============================================================"
  echo "Trace ${num}"
  echo "============================================================"
  diff <(sed '/^pid/d;s/] ([0-9]*)/] (PID)/g' ref/trace${num}.txt) <(sed '/^pid/d;s/] ([0-9]*)/] (PID)/g' sub/trace${num}.txt)
  if [[ $? -eq 0 ]]
  then
    correct=$((correct + 1))
  else
    echo "============================================================"
    echo "(-${POINTS}) Trace #${num} differs"
  fi
  echo ""
  echo "============================================================"
  echo ""
  echo ""
  echo ""
done

echo "============================================================"
echo "Trace 11"
echo "============================================================"

trace11pid_ref=$(sed -r '/ Job /!d;s/^[^(]*.([0-9]+).*/\1/' ref/trace11.txt)
trace11pid_sub=$(sed -r '/ Job /!d;s/^[^(]*.([0-9]+).*/\1/' sub/trace11.txt)

trace11count_ref=$(grep -E -c ^${trace11pid_ref} ref/trace11.txt)
trace11count_sub=$(grep -E -c ^${trace11pid_sub} sub/trace11.txt)

if [[ ${trace11count_ref} -eq ${trace11count_sub} ]]
then
  correct=$((correct + 1))
else
  echo "(-${POINTS}) Trace #${num} differs"
fi

echo ""
echo "============================================================"
echo ""
echo ""
echo ""


echo "============================================================"
echo "Trace 12"
echo "============================================================"


trace12pid=$(sed '/stopped by signal 20/!d;s/^[^(]*.\([0-9]*\).*/\1/' sub/trace12.txt)
trace12child=$((trace12pid + 1))

t12parent_stopped=$(grep -E "${trace12pid}.*\bT\b" -c sub/trace12.txt)
echo "  T12: did the parent stop? ${t12parent_stopped}"
t12child_stopped=$(grep -E "${trace12child}.*\bT\b" -c sub/trace12.txt)
echo "  T12: did the child stop?  ${t12child_stopped}"

t12_bothStopped=$((t12parent_stopped + t12child_stopped))

if [[ ${t12_bothStopped} -eq 2 ]]
then
  correct=$((correct + 1))
else
  echo "(-${POINTS}) Trace #12 didn't have the process group stop"
fi

echo ""
echo "============================================================"
echo ""
echo ""
echo ""



echo "============================================================"
echo "Trace 13"
echo "============================================================"

trace13pid=$(sed '/stopped by signal 20/!d;s/^[^(]*.\([0-9]*\).*/\1/' sub/trace13.txt)
trace13child=$((trace13pid + 1))

t13parent_stopped=$(grep -E "${trace13pid}.*\bT\b" -c sub/trace13.txt)
echo "  T13: did the parent stop? ${t13parent_stopped}"
t13child_stopped=$(grep -E "${trace13child}.*\bT\b" -c sub/trace13.txt)
echo "  T13: did the child stop?  ${t13child_stopped}"

t13_bothStopped=$((t13parent_stopped + t13child_stopped))
t13_childOnce=$(grep -E "\b${trace13child}\b" -c sub/trace13.txt)
echo "  T13: did the child and parent complete?   ${t13_childOnce}"

if [[ ${t12_bothStopped} -eq 2 ]] && [[ ${t13_childOnce} -eq 1 ]]
then
  correct=$((correct + 1))
else
  echo "(-${POINTS}) Trace #13 didn't have the process group resume to completion"
fi

echo ""
echo "============================================================"
echo ""
echo ""
echo ""



echo "============================================================"
echo "Trace 14"
echo "============================================================"


diff <(sed '/^pid/d;s/] ([0-9]*)/] (PID)/g' ref/trace14.txt) <(sed '/^pid/d;s/] ([0-9]*)/] (PID)/g' sub/trace14.txt)
if [[ $? -eq 0 ]]
then
  correct=$((correct + 1))
else
  echo "(-${POINTS}) Trace #14 doesn't have correct error handling messages"
fi

echo ""
echo "============================================================"
echo ""
echo ""
echo ""


echo "============================================================"
echo "Trace 15"
echo "============================================================"

diff <(sed '/^pid/d;s/] ([0-9]*)/] (PID)/g' ref/trace15.txt) <(sed '/^pid/d;s/] ([0-9]*)/] (PID)/g' sub/trace15.txt)
if [[ $? -eq 0 ]]
then
  correct=$((correct + 1))
else
  echo "(-${POINTS}) Trace #15 differs"
fi

echo ""
echo "============================================================"
echo ""
echo ""
echo ""



echo "============================================================"
echo "Trace 16"
echo "============================================================"


diff <(sed '/^pid/d;s/] ([0-9]*)/] (PID)/g' ref/trace16.txt) <(sed '/^pid/d;s/] ([0-9]*)/] (PID)/g' sub/trace16.txt)
if [[ $? -eq 0 ]]
then
  correct=$((correct + 1))
else
  echo "(-${POINTS}) Trace #16 differs"
fi

echo ""
echo "============================================================"
echo ""
echo ""
echo ""
echo "Score possible:  $((16 * ${POINTS}))"
echo "Points deducted: $(((16-correct)*${POINTS}))"
echo "Score earned:    $((${POINTS} * correct))"

score=$((${POINTS} * correct))

echo ""
echo "{ \"scores\": {\"auto\": ${score}}, \"scoreboard\": [\"${score}\"] }"

